import { IStringDictionary } from "@codingame/monaco-vscode-api/vscode/vs/base/common/collections";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { IUriIdentityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service";
import { IUserDataProfile, ProfileResourceType } from "@codingame/monaco-vscode-api/vscode/vs/platform/userDataProfile/common/userDataProfile";
import { IUserDataProfileStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/userDataProfile/common/userDataProfileStorageService.service";
import { ITreeItemCheckboxState, TreeItemCollapsibleState } from "@codingame/monaco-vscode-api/vscode/vs/workbench/common/views";
import { IProfileResource, IProfileResourceChildTreeItem, IProfileResourceInitializer, IProfileResourceTreeItem } from "@codingame/monaco-vscode-e72c94ca-257a-5b75-8b68-5a5fa3c18255-common/vscode/vs/workbench/services/userDataProfile/common/userDataProfile";
interface IGlobalState {
    storage: IStringDictionary<string>;
}
export declare class GlobalStateResourceInitializer implements IProfileResourceInitializer {
    private readonly storageService;
    constructor(storageService: IStorageService);
    initialize(content: string): Promise<void>;
}
export declare class GlobalStateResource implements IProfileResource {
    private readonly storageService;
    private readonly userDataProfileStorageService;
    private readonly logService;
    constructor(storageService: IStorageService, userDataProfileStorageService: IUserDataProfileStorageService, logService: ILogService);
    getContent(profile: IUserDataProfile): Promise<string>;
    apply(content: string, profile: IUserDataProfile): Promise<void>;
    getGlobalState(profile: IUserDataProfile): Promise<IGlobalState>;
    private writeGlobalState;
}
export declare abstract class GlobalStateResourceTreeItem implements IProfileResourceTreeItem {
    private readonly resource;
    private readonly uriIdentityService;
    readonly type = ProfileResourceType.GlobalState;
    readonly handle = ProfileResourceType.GlobalState;
    readonly label: {
        label: string;
    };
    readonly collapsibleState = TreeItemCollapsibleState.Collapsed;
    checkbox: ITreeItemCheckboxState | undefined;
    constructor(resource: URI, uriIdentityService: IUriIdentityService);
    getChildren(): Promise<IProfileResourceChildTreeItem[]>;
    abstract getContent(): Promise<string>;
    abstract isFromDefaultProfile(): boolean;
}
export declare class GlobalStateResourceExportTreeItem extends GlobalStateResourceTreeItem {
    private readonly profile;
    private readonly instantiationService;
    constructor(profile: IUserDataProfile, resource: URI, uriIdentityService: IUriIdentityService, instantiationService: IInstantiationService);
    hasContent(): Promise<boolean>;
    getContent(): Promise<string>;
    isFromDefaultProfile(): boolean;
}
export declare class GlobalStateResourceImportTreeItem extends GlobalStateResourceTreeItem {
    private readonly content;
    constructor(content: string, resource: URI, uriIdentityService: IUriIdentityService);
    getContent(): Promise<string>;
    isFromDefaultProfile(): boolean;
}
export {};
